{-# LANGUAGE GADTs, GeneralizedNewtypeDeriving #-}
{-# OPTIONS -Wall -fno-warn-missing-signatures #-}
module Language.Barracuda.Compiler.Infrastructure
  ( CompileConfig  (..)
  , CudaConfig     (..)
  , CppConfig      (..)
  , CompileTarget  (..)
  , defaultCudaConfig
  , defaultCppConfig
  , Compile
  , runCompile
  , uniqueId
  -- , getConfig
  , getTarget
  , useSharedMemory
  ) where

import Language.Barracuda.AST (Id)
import qualified Util.Unique as Unique

import Control.Monad.Trans (lift)
import qualified Control.Monad.Reader as Reader

------------------------------------------------------------------------
-- Compiler Infrastructure
------------------------------------------------------------------------
-- | Compiler configuration options
data CompileConfig
  = CudaTarget CudaConfig
  | CppTarget CppConfig

data CudaConfig
  = CudaConfig {
      sharedMemoryOptimization :: Bool
    }

data CppConfig
  =  CppConfig

defaultCudaConfig :: CompileConfig
defaultCudaConfig = CudaTarget $ CudaConfig {
                      sharedMemoryOptimization = False
                    }

defaultCppConfig :: CompileConfig
defaultCppConfig = CppTarget CppConfig

-- | Compilation target
data CompileTarget = Cpp | Cuda
  deriving (Eq, Ord, Show)

-- | A monad for compilation.  This provides the ability to get unique
-- IDs as well as the ability to intern array constants.
newtype Compile a =
  Compile ((Reader.ReaderT CompileConfig
              (Unique.Unique Int)) a)
  deriving (Monad)

runCompile :: CompileConfig -> Compile a -> a
runCompile conf (Compile r) = res
  where
    res = Unique.runUnique u [0..]
    u = Reader.runReaderT r conf

unique = lift Unique.unique

-- | Get a unique name with the specified prefix.
uniqueId :: String -> Compile Id
uniqueId prefix = Compile $ do
  n <- unique
  return (prefix ++ show n)

-- getConfig :: Compile CompileConfig
-- getConfig = Compile Reader.ask

getTarget :: Compile CompileTarget
getTarget = Compile $ do
  conf <- Reader.ask
  case conf of
    CudaTarget {} -> return Cuda
    CppTarget {}  -> return Cpp

useSharedMemory :: Compile Bool
useSharedMemory = Compile $ do
  conf <- Reader.ask
  case conf of
    CudaTarget c -> return $ sharedMemoryOptimization c
    CppTarget {} -> return False
