#ifndef ADD_ONE_2D_BENCHMARKS_HPP
#define ADD_ONE_2D_BENCHMARKS_HPP


#include "matrix_benchmark.hpp"
#include "test_array.hpp"
#include "random.hpp"

#include <cublas.h>
#include <cuda_runtime.h>


class cpu_add_one_2d_benchmark : public matrix_benchmark
{
protected:
    float_gen _generator;
    const float_matrix *_input;
    float_matrix *_result;

public:
    cpu_add_one_2d_benchmark (const char * outfile_name, rand_src &rng)
        : matrix_benchmark(outfile_name)
        , _generator(rng, 20.0f, 25.0f)
        , _input(NULL)
        , _result(NULL)
        {
        }

    virtual ~cpu_add_one_2d_benchmark ()
        {
        }
    
    virtual void setup (const unsigned rows, const unsigned cols)
        {
            _input = new float_matrix (
                rows,
                cols,
                new_test_array(_generator, rows * cols).get()
                );
            _result = new float_matrix(rows, cols);
        }

    virtual void run ()
        {
            cpu_add_one_2d(*_input, *_result);
        }

    virtual void teardown ()
        {
            delete _result;
            delete _input;
        }

    virtual const char * get_name () const
        {
            return "cpu_add_one_2d";
        }
};


class gpu_add_one_2d_benchmark : public matrix_benchmark
{
private:
    float_gen _generator;

    const gpu_float_matrix *_input;
    gpu_float_matrix *_result;

public:
    gpu_add_one_2d_benchmark (
        const char * outfile_name,
        rand_src &rng
        ) : matrix_benchmark(outfile_name)
          , _generator(rng, 20.0f, 25.0f)
          , _input(NULL)
          , _result(NULL)
        {
        }

    virtual ~gpu_add_one_2d_benchmark ()
        {
        }
    
    virtual void setup (const unsigned rows, const unsigned cols)
        {
            _input = new gpu_float_matrix (
                rows,
                cols,
                new_test_array(_generator, rows * cols).get()
                );
            _result = new gpu_float_matrix(rows, cols);
        }

    virtual void run ()
        {
            gpu_add_one_2d(*_input, *_result);
        }

    virtual void teardown ()
        {
            delete _result;
            delete _input;
        }

    virtual const char * get_name () const
        {
            return "gpu_add_one_2d";
        }
};


#endif
