#include "barracuda.hpp"
#include <cuda_runtime.h>


static __global__
void
jacobi_stencil (
    kernel_float_matrix in,
    kernel_float_matrix_view out
    )
{
    const int tx = threadIdx.x;
    const int ty = threadIdx.y;

    // compute ``global'' location in the arrays for the executing
    // thread.
    const int x = blockIdx.x * 16 + tx;
    const int y = blockIdx.y * 16 + ty;

    // only stencil the inner cells!
    if(0 < x && x < in.cols()-1 && 0 < y && y < in.rows()-1){
        out(y, x) = (in(y-1, x) +
                     in(y+1, x) +
                     in(y, x-1) +
                     in(y, x+1)) / 4.0f;
    }
}


void
handwritten_jacobi_stencil_noshmem (
    const gpu_float_matrix &input,
    gpu_float_matrix_view &output
    )
{
    const dim3 block_dims (16, 16, 1);
    const dim3 grid_dims (
        (input.cols() + 15) / 16,
        (input.rows() + 15) / 16,
        1
        );
    jacobi_stencil<<<grid_dims, block_dims>>> (
        kernel_float_matrix (input),
        kernel_float_matrix_view (output)
        );
}
