#include <csignal>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <unistd.h>

#include "timeout.hpp"

namespace {
    volatile bool KEEP_RUNNING = false;

    void alarm_action (int sig)
    {
        KEEP_RUNNING = false;
    }
} // end namespace


void timeout (unsigned seconds)
{
    struct sigaction sa;
    memset (&sa, '\0', sizeof(sa));
    sa.sa_handler = alarm_action;
    sigfillset (&sa.sa_mask);
    sigaction (SIGALRM, &sa, NULL);

    KEEP_RUNNING = true;
    unsigned err = alarm (seconds);
    if (err) {
        fprintf (
            stderr,
            "Alarm already set with %u remaining seconds\n",
            err
            );
        abort();
    }
}


void timeout_stop ()
{
    alarm (0);  // cancel any pending alarm request
    KEEP_RUNNING = false;
}


bool keep_running ()
{
    return KEEP_RUNNING;
}
