#ifndef BARRACUDA_HPP
#define BARRACUDA_HPP

#include <cmath>
#include <cassert>

#include "barracuda/host_and_device.hpp"
#include "barracuda/slice.hpp"

//------------------------------------------------------------
// 1D CPU arrays
//------------------------------------------------------------
#include "barracuda/array.hpp"
typedef array<float> float_array;
typedef array<int>   int_array;
typedef array<bool>  bool_array;

#include "barracuda/array_view.hpp"
typedef array_view<float> float_array_view;
typedef array_view<int>   int_array_view;
typedef array_view<bool>  bool_array_view;

//------------------------------------------------------------
// 1D GPU arrays
//------------------------------------------------------------
#ifndef NO_CUDA
#include "barracuda/gpu_array.hpp"
typedef gpu_array<float> gpu_float_array;
typedef gpu_array<int>   gpu_int_array;
typedef gpu_array<bool>  gpu_bool_array;

#include "barracuda/gpu_array_view.hpp"
typedef gpu_array_view<float> gpu_float_array_view;
typedef gpu_array_view<int>   gpu_int_array_view;
typedef gpu_array_view<bool>  gpu_bool_array_view;

#include "barracuda/kernel_array.hpp"
typedef kernel_array<float> kernel_float_array;
typedef kernel_array<int>   kernel_int_array;
typedef kernel_array<bool>  kernel_bool_array;

#include "barracuda/kernel_array_view.hpp"
typedef kernel_array_view<float> kernel_float_array_view;
typedef kernel_array_view<int>   kernel_int_array_view;
typedef kernel_array_view<bool>  kernel_bool_array_view;
#endif // NO_CUDA

//------------------------------------------------------------
// 2D CPU arrays
//------------------------------------------------------------
#include "barracuda/matrix.hpp"
typedef matrix<float> float_matrix;
typedef matrix<int>   int_matrix;
typedef matrix<bool>  bool_matrix;

#include "barracuda/matrix_view.hpp"
typedef matrix_view<float> float_matrix_view;
typedef matrix_view<int>   int_matrix_view;
typedef matrix_view<bool>  bool_matrix_view;

//------------------------------------------------------------
// 2D GPU arrays
//------------------------------------------------------------
#ifndef NO_CUDA
#include "barracuda/gpu_matrix.hpp"
typedef gpu_matrix<float> gpu_float_matrix;
typedef gpu_matrix<int>   gpu_int_matrix;
typedef gpu_matrix<bool>  gpu_bool_matrix;

#include "barracuda/gpu_matrix_view.hpp"
typedef gpu_matrix_view<float> gpu_float_matrix_view;
typedef gpu_matrix_view<int>   gpu_int_matrix_view;
typedef gpu_matrix_view<bool>  gpu_bool_matrix_view;

#include "barracuda/kernel_matrix.hpp"
typedef kernel_matrix<float> kernel_float_matrix;
typedef kernel_matrix<int>   kernel_int_matrix;
typedef kernel_matrix<bool>  kernel_bool_matrix;

#include "barracuda/kernel_matrix_view.hpp"
typedef kernel_matrix_view<float> kernel_float_matrix_view;
typedef kernel_matrix_view<int>   kernel_int_matrix_view;
typedef kernel_matrix_view<bool>  kernel_bool_matrix_view;
#endif // NO_CUDA

//------------------------------------------------------------
// Utility Procedures
//------------------------------------------------------------
#include "barracuda/utility_functions.hpp"

#endif // BARRACUDA_HPP
