#ifndef BARRACUDA_GPU_ARRAY_HPP
#define BARRACUDA_GPU_ARRAY_HPP

#include "host_and_device.hpp"
#include "utility_functions.hpp"

#include <cuda_runtime.h>

#include <cassert>
#include <algorithm>

template <class T>
class gpu_array
{
public:
  // -----------------------------------------------------------
  // Constructors / Destructor
  // -----------------------------------------------------------
  gpu_array()
    : _length(0)
  {
    cudaMalloc((void **)&_store, _length * sizeof(T));
  }

  gpu_array(int length)
    : _length(length)
  {
    cudaMalloc((void **)&_store, _length * sizeof(T));
  }

  gpu_array(int length, const T *values)
    : _length(length)
  {
    cudaMalloc((void **)&_store, _length * sizeof(T));
    cudaMemcpy(_store, values, _length * sizeof(T), cudaMemcpyHostToDevice);
  }

  ~gpu_array()
  {
    cudaFree(_store);
  }

  // // -----------------------------------------------------------
  // // Indexing
  // // -----------------------------------------------------------
  // const T & operator [](int idx) const
  // {
  //   assert(idx < _length);
  //   return _store[idx];
  // }

  // T & operator [](int idx)
  // {
  //   assert(idx < _length);
  //   return _store[idx];
  // }

  // -----------------------------------------------------------
  // Accessors
  // -----------------------------------------------------------
  int length() const
  {
    return _length;
  }

  const T * raw_array() const
  {
    return _store;
  }

  T * raw_array()
  {
    return _store;
  }

private:
  gpu_array<T> & operator = (const array<T> &);
  gpu_array(const array<T> &);

private:
  const int _length;
  T *_store;
};

#endif
