#ifndef GPU_ARRAY_VIEW_HPP
#define GPU_ARRAY_VIEW_HPP

template <class T>
class gpu_array_view
{
public:
    inline
    gpu_array_view (
        gpu_array<T> &arr
        ) : _array(arr)
          , _slice_info(0, arr.length(), 1)
        {
        }

    inline
    gpu_array_view (
        gpu_array<T> &arr,
        const slice slice_info
        ) : _array(arr)
          , _slice_info(slice_info)
        {
            assert(_slice_info.stride() != 0);
            assert(_slice_info.start() <= arr.length());
            assert(_slice_info.stop() <= arr.length());
        }

    HOST_AND_DEVICE
    inline
    int length () const
    {
        return _slice_info.length();
    }

// private:
    gpu_array<T> &_array;
    const slice _slice_info;
};

#endif // GPU_ARRAY_VIEW_HPP
