#ifndef GPU_MATRIX_VIEW_HPP
#define GPU_MATRIX_VIEW_HPP

template <class T>
class gpu_matrix_view
{
public:
    inline
    gpu_matrix_view (
        gpu_matrix<T> &mat
        ) : _matrix(mat)
          , _row_slice(0, mat.rows(), 1)
          , _col_slice(0, mat.cols(), 1)
        {
        }

    inline
    gpu_matrix_view (
        gpu_matrix<T> &mat,
        const slice row_slice,
        const slice col_slice
        ) : _matrix(mat)
          , _row_slice(row_slice)
          , _col_slice(col_slice)
        {
            assert(_row_slice.stride() != 0);
            assert(_row_slice.start() <= mat.rows());
            assert(_row_slice.stop() <= mat.rows());
            assert(_col_slice.stride() != 0);
            assert(_col_slice.start() <= mat.cols());
            assert(_col_slice.stop() <= mat.cols());
        }

    HOST_AND_DEVICE
    inline
    int rows () const
    {
        return _row_slice.length();
    }

    HOST_AND_DEVICE
    inline
    int cols () const
    {
        return _col_slice.length();
    }

// private:
    gpu_matrix<T> &_matrix;
    const slice _row_slice;
    const slice _col_slice;
};

#endif // GPU_MATRIX_VIEW_HPP
