#ifndef BARRACUDA_KERNEL_ARRAY_HPP
#define BARRACUDA_KERNEL_ARRAY_HPP

#include "gpu_array.hpp"

#include "host_and_device.hpp"
#include "utility_functions.hpp"

template <class T>
class kernel_array
{
public:
    // -----------------------------------------------------
    // Constructors / Destructor
    // -----------------------------------------------------
    inline
    kernel_array ( const gpu_array<T> &arr )
        : _length (arr.length())
        , _store  (const_cast<T *>(arr.raw_array()))
        {
        }

    // -----------------------------------------------------
    // Indexing
    // -----------------------------------------------------
    inline
    HOST_AND_DEVICE
    T operator [] (int idx) const
    {
      return _store[idx];
    }

    inline
    HOST_AND_DEVICE
    T & operator [] (int idx)
    {
      return _store[idx];
    }
  
    // -----------------------------------------------------
    // Accessors
    // -----------------------------------------------------
    inline
    HOST_AND_DEVICE
    int length () const
    {
      return _length;
    }

private:  
    const int _length;
    T *_store;
};


#endif
