#ifndef KERNEL_ARRAY_VIEW_HPP
#define KERNEL_ARRAY_VIEW_HPP

#include "host_and_device.hpp"
#include "utility_functions.hpp"
#include "kernel_array.hpp"
#include "gpu_array.hpp"
#include "gpu_array_view.hpp"

template <class T>
class kernel_array_view
{
public:
    inline
    kernel_array_view ( kernel_array<T> arr )
        : _array(arr)
        , _slice_info(0, arr.length(), 1)
        {
        }

    inline
    kernel_array_view (
        kernel_array<T> arr,
        const slice slice_info
        ) : _array(arr)
          , _slice_info(slice_info)
        {
            assert(_slice_info.stride() != 0);
            assert(_slice_info.start() <= arr.length());
            assert(_slice_info.stop() <= arr.length());
        }

    inline
    kernel_array_view ( gpu_array_view<T> &view )
        : _array(view._array)
        , _slice_info(view._slice_info)
        {
        }

    HOST_AND_DEVICE
    inline
    int length () const
    {
        return _slice_info.length();
    }

    // -----------------------------------------------------------
    // Indexing
    // -----------------------------------------------------------
    DEVICE
    inline
    const T & operator [](int idx) const
    {
        return _array[_slice_info(idx)];
    }

    DEVICE
    inline
    T & operator [] (int idx)
    {
        return _array[_slice_info(idx)];
    }

private:
    kernel_array<T> _array;
    const slice _slice_info;
};

#endif // KERNEL_ARRAY_VIEW_HPP
