module Main (main) where

import Language.Barracuda

-- | Forward difference of an array.
forwardDiff :: AExpr Float -> AExpr Float
forwardDiff x = azipWith (-) x'' x'
  where x'  = aslice x (0, alength x - 1, 1)
        x'' = aslice x (1, alength x, 1)

fd :: String -> Function
fd p = Function forwardDiff (p ++ "_forward_diff") ["x"] "result"

main :: IO ()
main = do
  compile
       defaultCppConfig
       ("cpu_forward_diff.hpp", "cpu_forward_diff.cpp")
       [fd "cpu"]
  compile
       defaultCudaConfig
       ("gpu_forward_diff_noshmem.hpp", "gpu_forward_diff_noshmem.cu")
       [fd "gpu_noshmem"]
  compile
       (CudaTarget CudaConfig { sharedMemoryOptimization = True })
       ("gpu_forward_diff_shmem.hpp", "gpu_forward_diff_shmem.cu")
       [fd "gpu_shmem"]
