#include "compare.hpp"


bool compare(float v1, float v2)
{
  if (!check_is_close(v1, v2, percent_tolerance(0.01f)))
    {
      fprintf(stderr,
              "different values: %.15f vs %.15f\n",
              v1, v2);
      return false;
    }
  else
    return true;
}


bool compare_arrays(const float_array &a1, const float_array &a2)
{
  assert(a1.length() == a2.length());
  for (unsigned i = 0; i < a1.length(); ++i)
  {
    if (!check_is_close(a1[i], a2[i], percent_tolerance(1.0f)))
    {
      fprintf(stderr,
              "different values at index %d with size %d: %.15f vs %.15f\n",
              i,
              a1.length(),
              a1[i],
              a2[i]);
      return false;
    }
  }
  return true;
}


bool compare_matrices(const float_matrix &a1, const float_matrix &a2)
{
  assert(a1.rows() == a2.rows());
  assert(a1.cols() == a2.cols());

  for (unsigned r = 0; r < a1.rows(); r += 1)
    for (unsigned c = 0; c < a1.cols(); c += 1)
      if (!check_is_close(a1(r, c), a2(r, c), percent_tolerance(1.0f)))
        {
          fprintf(stderr,
                  "different values at (%d, %d) "
                  "with rows of %d and cols of %d: %.15f vs %.15f\n",
                  r,
                  c,
                  a1.rows(),
                  a1.cols(),
                  a1(r, c),
                  a2(r, c));
          return false;
        }
  return true;
}
